/*
 * Accel_1.c
 *
 * Created: 06.06.2017 15:26:38
 * Author : steinegger
 */ 

#define F_CPU 16000000		//16 MHZ Crystal
#define BAUD 76800
#define UBRR (F_CPU / (16UL * BAUD)) - 1

#define ROMADDR_Resolution 1

#include <avr/io.h>
#include <util/delay.h>
#include <util/twi.h>
#include <stdbool.h>
#include <avr/eeprom.h>

#include "i2c_master.h"

void SendCh(unsigned char);
void SendStr(unsigned char[]);
void SendHex(uint8_t);

uint8_t ReadDataSet(int16_t*, int16_t*, int16_t*);
void SendDataSet(int16_t, int16_t, int16_t);

void SetResolution();

void RAM_ReadDataSet(int16_t*, int16_t*, int16_t*);
void RAM_StartSave();
void RAM_StartRead();
void RAM_Stop();
void RAM_SaveDataSet(int16_t, int16_t, int16_t);

double TIMESTEPMS = 1;			// measurement timesteps in ms
uint16_t MeasCounter = 0;
uint16_t MAXMeasurements = 10000;
bool b_MeasureActive = false;
uint16_t X,Y,Z;
uint8_t Resolution;


int main(void)
{
	DDRC |= (1<<PINC2);			// LED Output
	DDRC |= (0<<PINC1);			// BUTTON Input
    //PORTC |= (1<<PINC2);
	//init uart
	UBRR0H = (unsigned char)(UBRR>>8);
	UBRR0L = (unsigned char)UBRR;
	UCSR0B = (1<<RXEN0)|(1<<TXEN0);
	UCSR0C = (0<<USBS0)|(3<<UCSZ00);
	
	i2c_init();
	DDRB |= (1<<1)|(1<<2)|(1<<3)|(1<<5);	// SS,HOLD,MOSI and SCK as output
	PORTB |= ((1<<1)|(1<<2));				// SS and hold high
	SPCR = (1<<SPE)|(1<<MSTR)|(1<<1);		// SPI Enable as Master

	SetResolution();

    while (1) 
    {
		if(UCSR0A & (1<<RXC0)){				// Input from UART
			unsigned char CH_IN = UDR0;

			if(CH_IN == 'r'){														// TO DO ! ! ! ! ! MAKE INTERRUPT ! ! ! ! !
				SendCh('R'),
				SendCh(Resolution+48);
				SendStr("\r\n\0");

				RAM_StartRead();
				for(uint16_t i = 0; i<MeasCounter; i++){
					RAM_ReadDataSet(&X,&Y,&Z);
					SendDataSet(X,Y,Z);
				}
				RAM_Stop();
			}else{
				eeprom_write_byte(ROMADDR_Resolution, CH_IN-48);
				SetResolution();
			}
		}

		if(PINC & (1<<PINC1)){				// button gets pressed
			_delay_ms(10);
			PORTC |= (1<<PINC2);
			while(PINC & (1<<PINC1));
			_delay_ms(10);

			MeasCounter = 0;
			b_MeasureActive = true;
			RAM_StartSave();
		}

		while(b_MeasureActive == true){
			ReadDataSet(&X,&Y,&Z);
			RAM_SaveDataSet(X,Y,Z);
			_delay_ms(1);

			MeasCounter++;
			if((MeasCounter >= MAXMeasurements) || (PINC & (1<<PINC1))){
				b_MeasureActive = false;
				while(PINC & (1<<PINC1));
				PORTC &= ~(1<<PINC2);
				RAM_Stop();
				_delay_ms(100);
			}
		}		
		
    }
}

//-------------------------------------------------------------------------------

void RAM_ReadDataSet(int16_t *X, int16_t *Y, int16_t *Z){
	uint8_t HI,LO;

	SPDR = 0x00;
	while(!(SPSR & (1<<SPIF)));
	*X = SPDR;
	*X = (*X<<8);
	SPDR = 0x00;
	while(!(SPSR & (1<<SPIF)));
	*X += SPDR;


	SPDR = 0x00;
	while(!(SPSR & (1<<SPIF)));
	*Y = SPDR;
	*Y = (*Y<<8);
	SPDR = 0x00;
	while(!(SPSR & (1<<SPIF)));
	*Y += SPDR;

	SPDR = 0x00;
	while(!(SPSR & (1<<SPIF)));
	*Z = SPDR;
	*Z = (*Z<<8);
	SPDR = 0x00;
	while(!(SPSR & (1<<SPIF)));
	*Z += SPDR;
}

void RAM_StartRead(){
	PORTB &= ~(1<<2);		// SS low
	SPDR = 0x03;			// Transmit read instruction
	while(!(SPSR & (1<<SPIF)));
	SPDR = 0x00;			//Address 0x00__
	while(!(SPSR & (1<<SPIF)));
	SPDR = 0x00;			//Address 0x_00_
	while(!(SPSR & (1<<SPIF)));
	SPDR = 0x00;			//Address 0x__00
	while(!(SPSR & (1<<SPIF)));
}

void RAM_SaveDataSet(int16_t X, int16_t Y, int16_t Z){
	uint8_t HI = (X>>8);
	uint8_t LO = X;
	SPDR = HI;
	while(!(SPSR & (1<<SPIF)));
	SPDR = LO;
	while(!(SPSR & (1<<SPIF)));

	HI = (Y>>8);
	LO = Y;
	SPDR = HI;
	while(!(SPSR & (1<<SPIF)));
	SPDR = LO;
	while(!(SPSR & (1<<SPIF)));

	HI = (Z>>8);
	LO = Z;
	SPDR = HI;
	while(!(SPSR & (1<<SPIF)));
	SPDR = LO;
	while(!(SPSR & (1<<SPIF)));
}

void RAM_StartSave(){
	PORTB &= ~(1<<2);		// SS low
	SPDR = 0x02;			// Transmit write instruction
	while(!(SPSR & (1<<SPIF)));
	SPDR = 0x00;			//Address 0x00__
	while(!(SPSR & (1<<SPIF)));
	SPDR = 0x00;			//Address 0x_00_
	while(!(SPSR & (1<<SPIF)));
	SPDR = 0x00;			//Address 0x__00
	while(!(SPSR & (1<<SPIF)));
}

void RAM_Stop(){
	PORTB |= (1<<2);	// set SS high
}

//-------------------------------------------------------------------------------

void SetResolution(){
	
	uint8_t ReadRes = eeprom_read_byte(ROMADDR_Resolution);

	i2c_start(0x70+I2C_WRITE);
	i2c_write(0x14);
	i2c_start(0x70+I2C_READ);

	uint8_t ReadReg = i2c_read_ack();
	ReadReg &= (0b11100111);

	switch(ReadRes){
		case 2:	ReadReg |= (0b00000000);
				Resolution = 2;
				break;
		case 4: ReadReg |= (0b00001000); 
				Resolution = 4;
				break;
		case 8: ReadReg |= (0b00010000);
				Resolution = 8;
				break;
		default: Resolution = 2;
	}

	i2c_start(0x70+I2C_WRITE);
	i2c_write(0x14);
	i2c_write(ReadReg);
	return;
}

void SendDataSet(int16_t X, int16_t Y, int16_t Z){
	char str[6];
	SendCh('D');

	itoa(X,str,10);
	SendCh('X');
	SendStr(str);
	SendCh(';');

	itoa(Y,str,10);
	SendCh('Y');
	SendStr(str);
	SendCh(';');

	itoa(Z,str,10);
	SendCh('Z');
	SendStr(str);
	SendCh(';');

	SendStr("\r\n\0");
}

uint8_t ReadDataSet(int16_t *X, int16_t *Y, int16_t *Z){
	uint8_t error  = 0;

	error = i2c_start(0x70+I2C_WRITE);
	error = i2c_write(0x02);
	error = i2c_start(0x70+I2C_READ);

	uint8_t X_LO = i2c_read_ack();
	uint8_t X_HI = i2c_read_ack();
	uint8_t Y_LO = i2c_read_ack();
	uint8_t Y_HI = i2c_read_ack();
	uint8_t Z_LO = i2c_read_ack();
	uint8_t Z_HI = i2c_read_ack();

	X_LO = (X_LO >> 6);
	Y_LO = (Y_LO >> 6);
	Z_LO = (Z_LO >> 6);

	*X = ((int16_t)X_HI * 4) + X_LO;
	*Y = ((int16_t)Y_HI * 4) + X_LO;
	*Z = ((int16_t)Z_HI * 4) + X_LO;

	if((*X & (1<<9))){
		*X = *X | (0b1111110000000000);
	}

	if((*Y & (1<<9))){
		*Y = *Y | (0b1111110000000000);
	}

	if((*Z & (1<<9))){
		*Z = *Z | (0b1111110000000000);
	}

	return error;
}

//-------------------------------------------------------------------------------

void SendCh(unsigned char c){
	while ((UCSR0A & (1<<UDRE0)) == 0) {};
	UDR0 = c;
}

void SendStr(unsigned char s[])
{
	int i =0;
	
	while (s[i] != 0x00)
	{
		SendCh(s[i]);
		i++;
	}
}

void SendHex(uint8_t c){
	SendCh('0');
	SendCh('x');
	uint8_t LO = (c & 0x0F);
	uint8_t HI = (c >> 4);
	
	if(LO>=10){LO+=55;}else{LO+=48;}
	if(HI>=10){HI+=55;}else{HI+=48;}
	SendCh(HI);
	SendCh(LO);

	SendCh('\r');
	SendCh('\n');
}
